import React, { useState, useEffect } from 'react';
import ReactDOM from 'react-dom';
import ReactDropdown from 'react-dropdown';
import {
  csv,
  scaleLinear,
  scaleOrdinal,
  format,
  extent,
} from 'd3';
import { useData } from './useData';
import { AxisLeft } from './AxisLeft';
import { AxisBottom } from './AxisBottom';
import { Marks } from './Marks';
import { Dropdown } from './Dropdown';
import { ColorLegend } from './ColorLegend';

const width = window.innerWidth;
const menuHeight = 80;
const height = window.innerHeight - menuHeight;
const margin = {
  top: 20,
  right: 175,
  bottom: 65,
  left: 90,
};

const xAxisLabelOffset = 50;
const yAxisLabelOffset = 55;

const innerHeight = height - margin.top - margin.bottom;
const innerWidth = width - margin.left - margin.right;

const fadeOpacity = 0.2;

const App = () => {
  const data = useData();

  const initialXAttribute = 'petal_length';
  const [xAttribute, setXAttribute] = useState(
    initialXAttribute
  );
  const xValue = (d) => d[xAttribute];

  const initialYAttribute = 'sepal_width';
  const [yAttribute, setYAttribute] = useState(
    initialYAttribute
  );
  const yValue = (d) => d[yAttribute];

  const colorValue = (d) => d.species;
  const colorLegendLabel = 'Species';
  const circleRadius = 7;

  const [hoveredValue, setHoveredValue] = useState(null);
  
  if (!data) {
    return <pre>Loading...</pre>;
  }
  
  const filteredData = data.filter(d => hoveredValue === colorValue(d)); 
  
  const options = [];
  let row = {};
  data.columns.forEach((d) => {
    if (d === 'species') {
      return;
    }
    row = {
      value: d,
      label: d.split('_').map((d) => d[0].toUpperCase() + d.slice(1)).join(' '),
    };
    options.push(row);
  });

  const xAxisLabel = options.filter(
    (d) => d.value === xAttribute
  )[0].label;
  const yAxisLabel = options.filter(
    (d) => d.value === yAttribute
  )[0].label;
  const yAxisTickFormat = format('.2f');

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([0, innerHeight]);

  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(['#E6842A', '#137B80', '#8E6C8A']);

  return (
    <>
      <div className="menus-container">
        <span className="dropdown-label">X</span>
        <ReactDropdown
          options={options}
          value={xAttribute}
          onChange={({ value }) => setXAttribute(value)}
        />

        <span className="dropdown-label">Y</span>
        <ReactDropdown
          options={options}
          value={yAttribute}
          onChange={({ value }) => setYAttribute(value)}
        />
      </div>
      <svg width={width} height={height}>
        <g
          transform={`translate(${margin.left}, ${margin.top})`}
        >
          <AxisLeft
            yScale={yScale}
            tickFormat={yAxisTickFormat}
            innerWidth={innerWidth}
            tickOffset={5}
          />
          <text
            className="axis-label"
            textAnchor="middle"
            x={-yAxisLabelOffset}
            y={innerHeight / 2}
            style={{
              transformOrigin: `${-yAxisLabelOffset}px ${
                innerHeight / 2
              }px`,
            }}
            transform="rotate(-90)"
          >
            {yAxisLabel}
          </text>

          <AxisBottom
            xScale={xScale}
            innerHeight={innerHeight}
            tickOffset={5}
          />
          <text
            className="axis-label"
            x={innerWidth / 2}
            y={innerHeight + xAxisLabelOffset}
            textAnchor="middle"
          >
            {xAxisLabel}
          </text>

          <ColorLegend
            colorScale={colorScale}
            innerWidth={innerWidth}
            colorLegendLabel={colorLegendLabel}
            tickSpacing={22}
            tickSize={circleRadius}
            tickTextOffset={12}
            onHover={setHoveredValue}
            hoveredValue={hoveredValue}
            fadeOpacity={fadeOpacity}
          />

          <g opacity={hoveredValue ? fadeOpacity : 1}>
            <Marks
              data={data}
              xScale={xScale}
              xValue={xValue}
              yScale={yScale}
              yValue={yValue}
              colorScale={colorScale}
              colorValue={colorValue}
              tooltipFormat={yAxisTickFormat}
              circleRadius={circleRadius}
            />
          </g>
          <Marks
            data={filteredData}
            xScale={xScale}
            xValue={xValue}
            yScale={yScale}
            yValue={yValue}
            colorScale={colorScale}
            colorValue={colorValue}
            tooltipFormat={yAxisTickFormat}
            circleRadius={circleRadius}
          />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
